/*
 * A context resource offering support for opening files.
 *
 * Copyright (C) 2021, 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <ipc/cap_alloc.h>

#include "opener_context_resource.h"
#include "opener_context_object_server.h"
#include "opener_resource.h"

#include <string.h>



/* Support for providing access to files. */

OpenerContextResource::OpenerContextResource(OpenerResource *opener, Memory *memory)
: SimplePager(memory), _opener(opener)
{
}

ipc_server_default_config_type OpenerContextResource::config()
{
    return config_OpenerContextObject;
}



/* Opener context interface methods. */

long OpenerContextResource::mkdir(sys_mode_t mode)
{
    char *path = get_string();

    if (path == NULL)
        return -L4_EINVAL;

    return _opener->mkdir(path, mode);
}

long OpenerContextResource::open(flags_t flags, offset_t *size,
                                 l4_cap_idx_t *file, object_flags_t *object_flags)
{
    char *path = get_string();

    if (path == NULL)
        return -L4_EINVAL;

    return _opener->open(path, flags, size, file, object_flags);
}

long OpenerContextResource::remove()
{
    char *path = get_string();

    if (path == NULL)
        return -L4_EINVAL;

    return _opener->remove(path);
}

long OpenerContextResource::rename()
{
    char *source, *target;

    source = get_string();

    if (source == NULL)
        return -L4_EINVAL;

    target = get_string(strlen(source) + 1);

    if (target == NULL)
        return -L4_EINVAL;

    return _opener->rename(source, target);
}

long OpenerContextResource::stat()
{
    char *path = get_string();

    if (path == NULL)
        return -L4_EINVAL;

    return _opener->stat(path, (void *) _region->start, _region->size());
}

// vim: tabstop=4 expandtab shiftwidth=4
